unit InputFile;
(****************************************************************************
   Encapsulates TextFile for input


   Example of use
   --------------
   FIN := TInputFile.Open(Filename);   // create will set open flag
   try
     if FIN.IsOpen then begin
       while not FIN.EOF do begin
         S := FIN.NextLine;
         { process S }
         if BadNews then showmessage('Error on '+FIN.LineOfName+' Text:"'+Fin.LastLine+'"');
       end;
       FIN.Close;    // not necessary will be handled by FIN.free
     end else begin
       showmessage('Error:'+FIN.Emsg);
     end;
   finally
     FIN.free;
   end;
************************************************************************************)


interface

uses
    SysUtils;

type
    TInputFile = class(Tobject)
      Private
        F : Textfile;
        Linecount : integer;
        fOpen : boolean;
        fname : string;
      public
        LastLine : string;                      // What was the last line of text provided by NextLine
        EMsg: string;                           // Error message
        constructor Open(const Filename:string);
        destructor Destroy; override;
        procedure Close;                        // when we finish
        function EOF:boolean;                   // reached EOF?
        Function isOpen : boolean;              // open?
        function LineOfName : string;           // "Line n of file f" eg for reporting errors
        function NextLine:string;               // return next line of text
      end;

implementation

constructor TInputFile.Open(const Filename:string);
{ create and open file.
   isOpen function will tell if successfull
   Emsg may give report of problem }
begin
  inherited create;
  fOpen := false;
  LineCount := 0;
  lastline := '';

  if not fileexists(Filename) then begin
    Emsg := 'File "'+Filename+ '" not found';
    exit;
  end;

  try
    assignfile(F,Filename);
    reset(F);
    fOpen := true;
    fname := filename;
  except
    Emsg := 'Unable to open "'+Filename+ '"';
  end;
end;


destructor TInputFile.Destroy;
{ make sure we have closed the file}
begin
  if fopen then close;
  inherited destroy;
end;


function TInputFile.Close;
{ close the file }
begin
  try
    closefile(F);
  except
    ; // ignore any error
  end;
end;


function TInputFile.EOF:boolean;
{ Are we at the EOF ? }
begin
  result := system.eof(F);
end;


Function TInputFile.isOpen : boolean;
{ Is the file open }
begin
  result := fOpen;
end;


function TInputFile.LineOfName : string;
{ return a text in the form of Line nnn of file fff}
begin
  result := format('line %d of file "%s"',[LineCount,fname]);
end;


function TInputFile.NextLine:string;
{ read next line }
begin
  if eof then begin
    result := '';
  end else begin
    readln(F,LastLine);
    result := LastLine;
    linecount := linecount+1;
  end;
end;










end.
